<?php

namespace Drupal\mcapi_credcom\Element;

use Drupal\mcapi\Entity\Wallet;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element\Textfield;

/**
 * @FormElement("credcom_wallet")
 */
class CredcomWallet extends Textfield {

  /**
   * {@inheritdoc}
   */
  public function getInfo() {
    $info = parent::getInfo();
    $class = static::class;
    // This callback covers remote wallets only.
    $info['#autocomplete_route_name'] = 'mcapi_credcom.wallet.remote_autocomplete';
    $info['#placeholder'] = t('Remote wallet path');
    $info['#element_validate'] = [[$class, 'validate']];
    return $info;
  }


  /**
   * {@inheritdoc}
   */
  public static function __valueCallback(&$element, $input, FormStateInterface $form_state) {
    if ($input !== FALSE && is_array($input)) {
      $entity_ids = array_map(function (array $item) {
        return $item['target_id'];
      }, $input);
      $entities = \Drupal::entityTypeManager()->getStorage('mc_wallet')->loadMultiple($entity_ids);
      return static::getEntityLabels($entities);
    }
    return $input;
  }

  /**
   * Just as in Drupal\Core\Entity\Element\EntityAutocomplete we use te validate
   * callback to set the final value, so that the given value is still present
   * when the form has to be rebuilt.
   *
   * If the #value has a slash then this is a remote wallet which is not
   * validated but  replaced by the credcom wallet id, otherwise take the number
   * from the brackets.
   */
  public static function validate(array &$element, FormStateInterface $form_state, array &$complete_form) {
    if (strpos($element['#value'], '/') !== FALSE) {// Remote wallet
      $wid = \Drupal::config('mcapi_credcom.settings')->get('wallet_id');
      $field_id = end($element['#parents']);
      // There is no 'entries' value yet, so this must be merged.
      $form_state->set('remoteMetadata', [$wid => $element['#value']]);
      $form_state->setValueForElement($element, $wid);
    }
    else {
      // Identify a local wallet from the wallet id in brackets
      if(preg_match('/\(([0-9]+)\)/', $element['#value'], $matches))  {
        $wid = $matches[1];
      }
      // Or from the wallet id
      elseif (is_numeric($element['#value'])) {
        $wid = $element['#value'];
      }
      // And set the set the form_state value
      if (isset($wid) and Wallet::load($wid)) {
        $form_state->setValueForElement($element, $wid);
      }
      else {
        $form_state->setError($element, t('Unable to identify wallet wallet @id', ['@id' => $element['#value']]));
      }
    }
  }

}
