<?php

namespace Drupal\mcapi_credcom;

use Drupal\Core\Utility\Error;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpKernel\Event\ExceptionEvent;
use Symfony\Component\HttpKernel\KernelEvents;
use CreditCommons\Exceptions\CCFailure;

/**
 * Last-chance handler for exceptions: the final exception subscriber.
 */
class FinalExceptionSubscriber extends \Drupal\Core\EventSubscriber\FinalExceptionSubscriber implements EventSubscriberInterface {

  /**
   * Handles exceptions for this subscriber.
   *
   * @param \Symfony\Component\HttpKernel\Event\ExceptionEvent $event
   *   The event to process.
   */
  public function onException(ExceptionEvent $event) {
    $exception = $event->getThrowable();
    $error = Error::decodeException($exception);
    // @message, %function, %file, %line, 'backtrace', @backtrace_string, exception

    $backtrace_exception = $exception;
    while ($backtrace_exception->getPrevious()) {
      $backtrace_exception = $backtrace_exception->getPrevious();
    }
    $backtrace = $backtrace_exception->getTrace();
    // First trace is the error itself, already contained in the message.
    // While the second trace is the error source and also contained in the
    // message, the message doesn't contain argument values, so we output it
    // once more in the backtrace.
    //array_shift($backtrace);

    // Generate a backtrace containing only scalar argument values.
    $error['@backtrace'] = Error::formatBacktrace($backtrace);
    \Drupal::logger('credcom')->error('Drupal Exception: @message', ['@message' => $error['@message']]);
    $error = new CCFailure($error['@message']);
    // Overwrite the default backtrace which leads to here, not to where the error was
    $error->trace = $backtrace;
    $response = new JsonResponse(['errors' => [$error]], 500, ['Content-Type' => "application/json"]);
    $event->setResponse($response);
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() : array {
    // Run as the final (very late) KernelEvents::EXCEPTION subscriber.
    $events[KernelEvents::EXCEPTION][] = ['onException', -257];
    return $events;
  }
}
