<?php

namespace Drupal\mcapi_credcom\Form;

use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Form\FormStateInterface;

/**
 * Form to alter a workflow without changing its hashed value.
 */
class WorkflowTranslate extends EntityForm {

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);
    $w=0;
    $form['label'] = [
      '#title' => $this->t('Label'),
      '#type' => 'textfield',
      '#default_value' => $this->entity->label,
      '#weight' => $w++
    ];
    $form['summary'] = [
      '#title' => $this->t('Summary'),
      '#type' => 'textfield',
      '#default_value' => $this->entity->summary,
      '#weight' => $w++
    ];
    foreach($this->entity->states as $start_state => $info) {
      if (empty($info))continue;
      $form[$start_state] = [
        '#title' => $this->t('From state @name', ['@name' => $start_state]),
        '#type' => 'fieldset',
        '#weight' => $w++,
        '#tree' => TRUE
      ];
      foreach ($info as $end_state => $info2) {
        $form[$start_state][$end_state] = [
          '#title' => $this->t('To state @name', ['@name' => $end_state]),
          '#description' => $this->t('The name of the state transition (to put on the button).'),
          '#type' => 'textfield',
          '#default_value' => $info2['label'],
          '#weight' => $w++
        ];
      }
    }
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  protected function copyFormValuesToEntity(\Drupal\Core\Entity\EntityInterface $entity, array $form, FormStateInterface $form_state) {
    $values = $form_state->getValues();
    $entity->label = $values['label'];
    $entity->summary = $values['summary'];
    foreach ($entity->states as $from_state => $info1) {
      foreach ($info1 as $to_state => $info2) {
        $entity->states[$from_state][$to_state]['label'] = $values[$from_state][$to_state];
      }
    }
  }

}
