<?php

namespace Drupal\mcapi_credcom;

use Drupal\mcapi_credcom\Functions;
use Symfony\Component\HttpFoundation\Request;
use Drupal\Core\Access\AccessResult;
use CreditCommons\Exceptions\HashMismatchFailure;

/**
 * Access check for requests from trunkwards nodes.
 */
class HashChecker implements \Drupal\Core\Routing\Access\AccessInterface{

  /**
   * This is actually setting up drupal to respond like a credit commons node.
   */
  function __construct($route_match, $request_stack) {
    global $error_context;
    $config = \Drupal::Config('mcapi_credcom.settings');
    $error_context = (object)[
      'node' => $config->get('node_name'),
      // Headers are stored in lowercase but can be retrieved in any case.
      'user' => (string)$request_stack->getCurrentRequest()->headers->get('cc-user')
    ];
    if (substr($route_match->getRouteName(), 0, 13) == 'mcapi_credcom') {
      set_exception_handler(['\Drupal\mcapi_credcom\Functions', 'CredcomExceptionHandler']);
      set_error_handler(['\Drupal\mcapi_credcom\Functions', 'CredcomErrorHandler']);
    }
  }

  /**
   * Checks the hash passed in the headers against the hash in the database.
   *
   * @param RequestStack $request
   *
   * @return \Drupal\Core\Access\AccessResultInterface
   */
  public function access(Request $request) {
    global $error_context;
    $error_context->method = $request->getMethod();
    $error_context->path = substr($request->getPathInfo(), 8);
    $remote_hash = ($request->headers->get('cc-auth') == 'null') ? '' : (string)$request->headers->get('cc-auth');
    $local_hash = Functions::getlastHash();
    if ($request->headers->has('cc-user')) {
      if ($local_hash == $remote_hash) {
        return AccessResult::allowed();
      }
    }
    throw new HashMismatchFailure($error_context->user, $local_hash, $remote_hash);
  }

}
