<?php

namespace Drupal\mcapi_credcom\Plugin\views\field;

use Drupal\mcapi\Entity\Wallet;
use Drupal\mcapi\Entity\Transaction;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Entity\Exception\UndefinedLinkTemplateException;
use Drupal\Core\Entity\EntityMalformedException;
use Drupal\views\Plugin\views\field\EntityLabel;
use Drupal\views\ResultRow;
use Drupal\views\Plugin\views\display\DisplayPluginBase;
use Drupal\views\ViewExecutable;


/**
 * Field handler to display entity label optionally linked to entity page.
 *
 * @ViewsField("credcom_wallet_label")
 *
 * @note Pretty hacky coz I couldn't understand what the type field, required in
 * the field definition, does.
 */
class WalletLabel extends EntityLabel {

  protected $config;

  /**
   * {@inheritDoc}
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    parent::buildOptionsForm($form, $form_state);
    $form['link_to_entity']['#title'] = $this->t('Link to wallet');
    $form['link_to_entity']['#description'] = $this->t('Link the wallet name to its canonical page.');
  }


  public function init(ViewExecutable $view, DisplayPluginBase $display, array &$options = NULL) {
    if (!isset($this->definition['entity type field'])) {
      // I can't work out what this value is for.
      $this->definition['entity type field'] = 'wid';
    }
    $this->config = \Drupal::config('mcapi_credcom.settings');
    parent::init($view, $display, $options);
  }


  /**
   * {@inheritDoc}
   */
  public function render(ResultRow $values) {
    $entity = $values->_entity; // The main entity not the relationship entity.
    // In a transaction view the intertrading wallet must replace its name.
    if ($entity instanceOf Transaction) {
      $name = $this->getValue($values);// wallet name
      $entry_metadata = $entity->entries->first()->metadata;
      $wid = $this->config->get('wallet_id');
      if ($name == $this->config->get('wallet_name') and $remote_name = $entry_metadata[$wid]) {
        $parts = explode('/', $remote_name);
        $output = t('@name of @node', ['@name' => array_pop($parts), '@node' => array_pop($parts)]);
        $output = $this->sanitizeValue($output);
      }
      else{ // All other wallets
        $output = $name;
      }
    }
    elseif ($entity instanceOf Wallet) {
      if ($this->options['link_to_entity']) {
        try {
          $this->options['alter']['url'] = $entity->toUrl();
          $this->options['alter']['make_link'] = TRUE;
        }
        catch (UndefinedLinkTemplateException $e) {
          $this->options['alter']['make_link'] = FALSE;
        }
        catch (EntityMalformedException $e) {
          $this->options['alter']['make_link'] = FALSE;
        }
      }
      $output = $entity->label();
    }
    return $this->sanitizeValue($output);
  }

  /**
   * {@inheritDoc}
   */
  function getValue(ResultRow $values, $field = NULL) {

    if ($field == 'type') return 'mc_wallet';

    // $field is NULL, gives the wallet name.
    return parent::getValue($values, $field);
  }

}
