<?php

namespace Drupal\mcapi_forms\Plugin\Block;

use Drupal\mcapi_forms\Entity\McForm;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\mcapi\Entity\Transaction;

/**
 * Provides a designed transaction form.
 *
 * @Block(
 *   id = "mc_form",
 *   admin_label = @Translation("Transaction form"),
 *   category = @Translation("Community Accounting")
 * )
 */
class DesignedForm extends BlockBase {

  /**
   * {@inheritDoc}
   */
  public function build() {
    return \Drupal::service('entity.form_builder')
      ->getForm(
        Transaction::create(),
        $this->configuration['mcapiform']
      );
  }

  /**
   * {@inheritDoc}
   */
  public function access(AccountInterface $account, $return_as_object = FALSE) {
    $access = parent::access($account, TRUE);
    if ($access->allowed()) {
      $route_name = \Drupal::service('current_route_match')->getRouteName();
      // Block is available if the main page is not already a transaction form.
      if (substr($route_name, 0, 14) == 'mcapi.mc_form.') {
        $access = AccessResult::forbidden('Transaction form block does not show on the form page');
      }
      // Or an operation form)
      elseif (substr($route_name, 0, 22) == 'entity.mc_transaction.') {
        $access = AccessResult::forbidden('Transaction forms do not show on transaction operation pages.');
      }
    }
    return $return_as_object ? $access : $access->isAllowed();
  }

  /**
   * {@inheritDoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $options = [];
    foreach (McForm::loadMultiple() as $id => $mcapiForm) {
      $options[$id] = $mcapiForm->label();
    }
    $form['editform_type'] = [
      '#title' => t('Display'),
      '#type' => 'select',
      '#options' => $options,
      '#default_value' => $this->configuration['mode']??key($options),
    ];
    return $form;
  }

  /**
   * {@inheritDoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    $this->configuration['editform_type'] = $form_state->getValues('editform_type');
  }

}
