<?php

namespace Drupal\mcapi_forms\Plugin\migrate\process;

use Drupal\migrate\MigrateExecutableInterface;
use Drupal\migrate\ProcessPluginBase;
use Drupal\migrate\Row;
use Drupal\migrate\MigrateSkipRowException;

/**
 * Convert the form template
 *
 * @MigrateProcessPlugin(
 *   id = "d7_mcapi_form_type"
 * )
 */
class McapiFormType extends ProcessPluginBase {

  /**
   * {@inheritDoc}
   *
   * Old template is of the form:
   * Payer: [mcapiform:payer] Payee: [mcapiform:payee] [mcapiform:worth] for [mcapiform:description]
   *
   * $value should 1 or 3, i.e. the perspective from the d7 form
   * $output should be one of the default forms, standard_bill, standard_credit or 3rdparty
   */
  public function transform($value, MigrateExecutableInterface $migrate_executable, Row $row, $destination_property) {
    if ($value == 3) {
      $wf = '3rdparty';
    }
    elseif ($value == 1) {
      $dir = $row->getSourceProperty('direction')->preset;
      if ($dir == 'incoming') {
        $wf = 'default_bill';
      }
      elseif ($dir == 'outgoing') {
        $wf = 'default_credit';
      }
      else {
        throw new MigrateSkipRowException("Unrecognised direction $dir");
      }
    }
    return $wf;
  }

}
