<?php

namespace Drupal\mcapi_limits\Plugin\Validation\Constraint;

use Drupal\mcapi\Functions;
use Symfony\Component\Validator\ConstraintValidator;
use Symfony\Component\Validator\Constraint;

/**
 * Entity level constraint validator checking whether a wallet is beyond its limits.
 */
class TransactionLimitsConstraintValidator extends ConstraintValidator {

  /**
   * {@inheritdoc}
   */
  public function validate($transaction, Constraint $constraint) {
    // Curiously entity field validations runs even if the form field validation
    // fails. So skip if a wallet id is invalid.
    if (!$transaction->payee or !$transaction->payer) {
      return;
    }
    if ($transgressions = mcapi_limits_check($transaction)) {
      foreach ($transgressions as $wid => $transgression) {
        // These constraint properties are to send the message outside the site
        // i.e. for CreditCommons\Exceptions
        $constraint->wid = $transgression->wallet->id();
        $constraint->diff = $transgression->diff();
        // We can't know the path to the quant field unfortunately, so take a guess
        // @todo make a Function to find out the path
        $path = 'quant';
        $res = $this->context
          ->buildViolation($constraint->message)
          ->setParameter('%wallet', $transgression->wallet->label())
          ->setParameter('%limit', strip_tags(Functions::formatCurrency($transgression->limit(), FALSE)))
          ->setParameter('%excess', strip_tags(Functions::formatCurrency($transgression->diff(), FALSE)))
          ->atPath($path)
          ->addViolation();
      }
    }
  }

}
