<?php

namespace Drupal\mcapi\Controller;
use Drupal\mcapi\Entity\Transaction;
use Drupal\mcapi\Entity\Workflow;
use Drupal\system\Controller\SystemController;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Drupal\Core\Template\Attribute;

/**
 * Community accounting menu / admin page showing workflow info at the top.
 */
class AccountingAdmin extends SystemController {

  /**
   * {@inheritDoc}
   */
  public function systemAdminMenuBlockPage() {
    // Show a warning if there aren't enough wallets to trade
    $wallets = $this->entityTypeManager()->getStorage('mc_wallet')->getQuery()
      ->accessCheck(FALSE)
      ->count()->execute();
    if ($wallets < 2) {
      $message = $this->t("There aren't enough wallets for you to create a transaction. Wallets are at the same time as, or on the canonical page of, the entities that will hold them.");
      $this->messenger()->addWarning($message);
    }

    $renderable = [];
    $workflows = array_map(
      function ($wf) {
        $url = Url::fromRoute(
          'entity.mc_transaction.add_form',
          ['mc_workflow'=> $wf->id],
          ['attributes' => ['title' => $wf->summary]]
        );
        return t(
          'New @type transaction',
          ['@type' => Link::fromTextAndUrl($wf->label(), $url)->toString()]
        );
      },
      Workflow::loadAlphabetical()
    );
    //$workflows[] = Link::fromTextAndUrl(t('Manage'), Url::fromRoute('entity.mc_workflow.collection'))->toString();
    $renderable['types'] = [
      '#title' => $this->t('Transaction workflow types'),
      '#description' => $this->t('@todo local workflows should be editable.'),
      '#theme' => 'item_list',
      '#list_type' => 'ul',
      '#wrapper_attributes' => new Attribute(['style' => 'display:inline-block; vertical-align:top;']),
      '#items' => $workflows,
      '#weight' => -2
    ];
    $states = Transaction::states();
    unset($states['validated'], $states['temp']);
    $renderable['states'] = [
      '#title' => $this->t('Workflow states'),
      '#description' => $this->t('It is not currently possible to add new states.'),
      '#theme' => 'item_list',
      '#list_type' => 'ul',
      '#wrapper_attributes' => new Attribute(['style' => 'display:inline-block; margin-left:5em; vertical-align:top;']),
      '#items' => $states,
      '#weight' => -2
    ];

    $renderable[] = parent::systemAdminMenuBlockPage();
    return $renderable;
  }

}
