<?php

namespace Drupal\mcapi\Controller;

use Drupal\mcapi\Entity\Transaction;
use Drupal\Core\Controller\ControllerBase;

/**
 * Add a signature from a user who may not be logged in.
 *
 * Depends on a token generated in the 'signhere' mail.
 */
class AnonSign extends ControllerBase {

  /**
   * Sign the transaction and redirect to the canonical page.
   *
   * @param Transaction $transaction
   *   Transaction ID
   * @param int $uid
   *   User ID.
   * @param string $hash
   *   Hash which hopefully corresponds to #xid and $uid.
   */
  public function sign(Transaction $transaction, int $uid, string $hash) {
    try {
      $transaction->signatures->sign($uid);
      $transaction->save();
    }
    catch (\Exception $e) {
      $this->messenger()->addStatus(t('The signature was not needed, and had no effect.'));
    }

    return $this->redirect(
      'entity.mc_transaction.canonical',
      ['mc_transaction' => $transaction->id()]
    );

  }

}
