<?php

namespace Drupal\mcapi\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Url;
use Drupal\Core\Link;

/**
 * Builds a list of transaction forms.
 */
class FormList extends ControllerBase {

  /**
   * Build a table showing all the transaction forms.
   */
  public function buildPage() {
    $build['table'] = [
      '#type' => 'table',
      '#header' => [
        'title' => $this->t('Form name'),
        'path' => $this->t('Path'),
        'workflow' => $this->t('Type/workflow'),
        'operations' => $this->t('Form operations'),
      ],
      '#rows' => [],
    ];

    // Work out the menu links available for each path.
    foreach ($this->moduleHandler()->invokeAll('mcapi_form_list') as $form_name => $info) {
      $url = Url::fromRoute($info['route_name'], $info['route_parameters']??[]);
      $build['table']['#rows'][$form_name] = [
        'title' => $info['label'],
        'path' => Link::fromTextAndUrl($url->toString(), $url),
        'workflow' => $info['workflow'],
        'operations' => [
          'data' => [
            '#type' => 'operations',
            '#links' => $info['operations'],
          ],
        ],
      ];
    }
    return $build;
  }

}
