<?php

namespace Drupal\mcapi\Element;

use Drupal\mcapi\Entity\Wallet;
use Drupal\Core\Render\Element\Radios;
use Drupal\Core\Form\FormStateInterface;
/**
 * Form element to select the wallet to trade from. Hidden if only wallet is available.
 *
 * @RenderElement("my_wallet")
 */
class MyWallet extends Radios {

  /**
   * {@inheritDoc}
   */
  public static function processRadios(&$element, FormStateInterface $form_state, &$complete_form) {
    $uid = \Drupal::currentuser()->id();
    $wids = \Drupal::entityTypeManager()->getStorage('mc_wallet')->walletsOf($uid);
    if (count($wids) > 1) {
      foreach (Wallet::loadMultiple($wids) as $wid => $wallet) {
        $element['#options'][$wid] = $wallet->label();
      }
      $element['#title'] = 'My wallet';
      $element['#weight'] = -1;
      return parent::processRadios($element, $form_state, $complete_form);
    }
    else {
      return [
        '#name' => $element['#name'],
        '#type' => 'hidden', // 'value' doesn't seem to work.
        '#value' => reset($wids),
        '#parents' => [$element['#name']] // Needed for validation
      ];
    }
  }

  /**
   * {@inheritDoc}
   */
  public static function preRenderCompositeFormElement($element) {
    $element = parent::preRenderCompositeFormElement($element);
    $element['#theme_wrappers'] = [];
    return $element;
  }

}
