<?php

namespace Drupal\mcapi\Entity\Storage;

use Drupal\mcapi\Entity\Wallet;
use Drupal\mcapi\Entity\Transaction;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Entity\ContentEntityInterface;

/**
 * Storage controller for Transaction entity.
 */
class TransactionStorage extends TransactionIndexStorage {

  /**
   * {@inheritDoc}
   */
  public function resetCache(array $ids = NULL) {
    parent::resetCache($ids);
  }

  /**
   * {@inheritDoc}
   */
  protected function doDelete($entities) {
    // First of all we need to get a flat array of all the entities.
    foreach ($entities as $xid => $transaction) {
      $this->invalidateWalletCache($transaction);
    }
    parent::doDelete($entities);
  }

  /**
   * @param EntityInterface $entity
   * @param bool $update
   */
  protected function doPostSave(EntityInterface $entity, $update) {
    $this->invalidateWalletCache($entity);
    parent::doPostSave($entity, $update);
  }

  /**
   * @param Transaction $transaction
   */
  private function invalidateWalletCache(Transaction $transaction) {
    $tags = [];
    foreach (array_chunk($transaction->allWalletIds(), 50) as $wids) {
      foreach (Wallet::loadMultiple($wids) as $wallet) {
        $tags = array_merge($tags, $wallet->getCacheTagsToInvalidate());
      }
    }
    Cache::invalidateTags($tags);
  }

  /**
   * {@inheritDoc}
   */
  public function getQueryServiceName() {
    return 'mc_transaction.query.sql';
  }

  /**
   * Get a list all the revisions.
   * @param int $xid
   * @return int[]
   */
  public function revisionIds(int $xid) {
    return $this->database->select('mc_transaction_revision', 'cr')
      ->fields('cr', ['vxid'])
      ->condition('xid', $xid)
      ->orderBy('vxid')
      ->execute()
      ->fetchCol();
  }

  /**
   * {@inheritDoc}
   */
  public function firstCreated() : int {
    $result = $this->database->select('mc_transaction', 't')
      ->fields('t', ['created'])
      ->condition('state', 'completed')
      ->orderBy('created', 'DESC')
      ->range(0, 1)
      ->execute();

    if ($obj = $result->fetch()) {
      return $obj->created;
    }
    return 0;
  }

  /**
   * {@inheritDoc}
   */
  protected function initFieldValues(ContentEntityInterface $entity, array $values = [], array $field_names = []) {
    foreach (['payer', 'payee', 'quant', 'description'] as $prop) {
      $suffix = ($prop == 'payer' or $prop == 'payee') ? '_wid' : '';
      if (isset($values[$prop]) and empty($values['entries'][$prop.$suffix])) {
        $values['entries'][$prop.$suffix] = $values[$prop];
        unset($values[$prop]);
      }
    }
    parent::initFieldValues($entity, $values, $field_names);
  }

}
