<?php

namespace Drupal\mcapi\Entity\ViewBuilder;

use Drupal\mcapi\Entity\Transaction;
use Drupal\Core\Render\Markup;
use Drupal\Core\Url;
use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Routing\CurrentRouteMatch;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Entity\EntityViewBuilder;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Render controller for transactions.
 */
class TransactionViewBuilder extends EntityViewBuilder {

  protected $settings;
  protected $routeMatch;

  /**
   *
   * @param type $entity_type
   * @param type $entity_repository
   * @param type $language_manager
   * @param type $theme_registry
   * @param type $entity_display_repository
   * @param ConfigFactory $config_factory
   * @param CurrentRouteMatch $route_match
   * @param ModuleHandler $module_handler
   */
  public function __construct($entity_type, $entity_repository, $language_manager, $theme_registry, $entity_display_repository, ConfigFactory $config_factory, CurrentRouteMatch $route_match) {
    parent::__construct($entity_type, $entity_repository, $language_manager, $theme_registry, $entity_display_repository);
    $this->settings = $config_factory->get('mcapi.settings');
    $this->routeMatch = $route_match;
  }

  /**
   * {@inheritDoc}
   *
   * @todo update with entity_type.manager when core interface changes
   */
  public static function createInstance(ContainerInterface $container, EntityTypeInterface $entity_type) {
    return new static(
      $entity_type,
      $container->get('entity.repository'),
      $container->get('language_manager'),
      $container->get('theme.registry'),
      $container->get('entity_display.repository'),
      $container->get('config.factory'),
      $container->get('current_route_match')
    );
  }

  /**
   * {@inheritDoc}
   */
  protected function getBuildDefaults(EntityInterface $entity, $view_mode) {
    $build = parent::getBuildDefaults($entity, $view_mode);

    $build += [
      '#attributes' => [
        'class' => [
          'mc-transaction',
          'type-' . $entity->workflow->target_id,
          'state-' . $entity->state->value,
        ]
      ],
      '#attached' => [
        // For some reason in Renderer::updatestack, this bubbles up twice.
        'library' => ['mcapi/transaction'],
      ],
    ];
    switch ($view_mode) {
      case 'full':
        $build['#links'] = [];
        if ($this->routeMatch->getRouteName() == 'entity.mc_transaction.canonical') {
          $build['#links'] = $this->linkList($entity);
        }
        $build['#watermark'] = $entity->state->value == 'pending' ? t('Pending') : '';
        $build['#attributes']['class'][] = 'certificate';
        break;

      case 'sentence':
        // Do NOT user this to render a child.
        $template = $this->settings->get('sentence_template') . ' [xaction:links]';
        $sentence =  \Drupal::token()->replace($template, ['xaction' => $entity]);
        $build += [
          '#prefix' => '<div class = "transaction-sentence">',
          '#markup' => Markup::create($sentence . $this->linkList($entity)),
          '#suffix' => '</div>'
        ];
        $build['#attributes']['class'][] = 'sentence';
    }
    // @todo we might need to use the post-render cache to get the links right instead of template_preprocess_mc_transaction
    return $build;
  }

  /**
   * Make a renderable array for transaction operations
   *
   * @param Transaction $transaction
   *
   * @return array
   *   A renderable array.
   */
  private function linkList(Transaction $transaction) : array {
    $links = \Drupal::entityTypeManager()
      ->getListBuilder('mc_transaction')
      ->getOperations($transaction);
    // $the links are in order
    return ['#theme' => 'links', '#links' => $links];
  }

}
