<?php

namespace Drupal\mcapi\Entity\ViewBuilder;

use Drupal\Core\Entity\EntityViewBuilder;
use Drupal\Core\Url;

/**
 * Render controller for wallets.
 * @todo inject config
 */
class WalletViewBuilder extends EntityViewBuilder {

  /**
   * {@inheritDoc}
   */
  public function buildComponents(&$build, array $entities, array $displays, $view_mode) {
    $display = $displays['mc_wallet'];
    foreach ($entities as $id => $wallet) {
      if ($view_mode <> 'full') {
        // Show the wallet title at the top
        $build[$id]['title'] = [
          '#type' => 'item',
          '#markup'=> '<strong>'. $wallet->label(). '('. $wallet->id() .')</strong>',
          '#weight' => -2,
        ];
      }
      if ($options = $display->getComponent('balance_bars')) {
        $build[$id]['balance_bars'] = mc_wallet_view_balance_bars($wallet) + [
          '#title' => $this->t('Given and received'),
          '#weight' => $options['weight'],
          '#theme_wrappers' => ['mc_wallet_component']
        ];
      }
      if ($options = $display->getComponent('balance_viz')) {
        $build[$id]['balance_viz'] = mc_wallet_view_balance_viz($wallet) + [
          '#title' => $this->t('Balance'),
          '#weight' => $options['weight'],
          '#theme_wrappers' => ['mc_wallet_component']
        ];
      }
      if ($options = $display->getComponent('stats')) {
        $build[$id]['stats'] = mc_wallet_view_stats($wallet) + [
          '#title' => $this->t('Summary stats'),
          '#weight' => $options['weight'],
          '#theme_wrappers' => ['mc_wallet_component']
        ];
      }
      if ($options = $display->getComponent('histories')) {
        $build[$id]['histories'] = mc_wallet_view_balance_history($wallet, 300, 150) + [
          '#title' => $this->t('Balance history'),
          '#weight' => $options['weight'],
          '#theme_wrappers' => ['mc_wallet_component']
        ];
      }
      if ($options = $display->getComponent('link_transactions')) {
        $view_name = \Drupal::config('mcapi.settings')->get('user_transaction_view');
        $build[$id]['transactions'] = [
          '#type' => 'link',
          '#title' => t('Transactions...'),
          '#url' => Url::fromRoute('view.'.$view_name.'.wallet_page', ['arg_0' => $wallet->id()]),
          '#weight' => $options['weight'],
          '#attributes' => ['class' => ['component', 'link']]
        ];
      }
      if ($view_mode <> 'full') {
        $build[$id]['canonical'] = self::canonicalLink($wallet, 20);
      }
    }
  }

  static function canonicalLink(\Drupal\mcapi\Entity\WalletInterface $wallet, int $weight) {
    return[
      '#type' => 'link',
      '#title' => t('View wallet...'),
      '#url' => Url::fromRoute('entity.mc_wallet.canonical', ['mc_wallet' => $wallet->id()]),
      '#weight' => $weight,
      '#attributes' => ['class' => ['component', 'link']]
    ];
  }

}
