<?php

namespace Drupal\mcapi\Entity\Views;

use Drupal\views\EntityViewsData;

/**
 * Views data for wallet entity.
 */
class WalletViewsData extends EntityViewsData {

  /**
   * {@inheritDoc}
   */
  public function getViewsData() {
    $data = parent::getViewsData();
    $data['mc_wallet']['balance'] = [
      'title' => $this->t('Balance'),
      'help' => $this->t("Sum of all this wallet's credits minus debits"),
      'field' => [
        'id' => 'wallet_stat',
        'stat' => 'balance'
      ]
    ];
    $data['mc_wallet']['volume'] = [
      'title' => $this->t('Volume of transactions'),
      'help' => $this->t("Sum of all this wallet's transactions"),
      'field' => [
        'id' => 'mc_quant',
        'stat' => 'volume'
      ]
    ];
    $data['mc_wallet']['gross_in'] = [
      'title' => $this->t('Gross income'),
      'help' => $this->t("Sum of all this wallet's income ever"),
      'field' => [
        'id' => 'mc_quant',
        'stat' => 'gross_in'
      ]
    ];
    $data['mc_wallet']['gross_out'] = [
      'title' => $this->t('Gross expenditure'),
      'help' => $this->t("Sum of all this wallet's outgoings ever"),
      'field' => [
        'id' => 'mc_quant',
        'stat' => 'gross_out'
      ]
    ];
    $data['mc_wallet']['trades'] = [
      'title' => $this->t('Number of trades'),
      'help' => $this->t("Number of transactions involving this wallet"),
      'field' => [
        'id' => 'field',
        'stat' => 'trades'
      ]
    ];
    $data['mc_wallet']['partners'] = [
      'title' => $this->t('Partner count'),
      'help' => $this->t("Number of unique trading partners"),
      'field' => [
        'id' => 'field',
        'stat' => 'partners'
      ]
    ];

    // This allows us to do group queries such as find the balance per wallet.
    $data['mc_transactions_index']['table']['join'] = [
      'mc_wallet' => [
        'left_field' => 'wid',
        'field' => 'wallet_id',
        'required' => TRUE
      ],
    ];

    return $data;
  }
}
