<?php

namespace Drupal\mcapi\Entity;

use Drupal\Core\Config\Entity\ConfigEntityBase;

/**
 * Defines the transaction workflow entity class, i.e. the workflow definition.
 *
 * @ConfigEntityType(
 *   id = "mc_workflow",
 *   label = @Translation("Transaction workflow"),
 *   config_prefix = "mc_workflow",
 *   admin_permission = "configure mcapi",
 *   entity_keys = {
 *     "id" = "id",
 *     "label" = "label",
 *   },
 *   handlers = {
 *     "form" = {
 *       "default" = "Drupal\mcapi\Form\WorkflowEdit",
 *       "delete" = "Drupal\Core\Entity\EntityDeleteForm"
 *     },
 *     "route_provider" = {
 *       "html" = "Drupal\Core\Entity\Routing\DefaultHtmlRouteProvider",
 *     },
 *     "list_builder" = "Drupal\mcapi\Entity\ListBuilder\WorkflowListBuilder"
 *   },
 *   links = {
 *     "add-form" = "/admin/accounting/workflow/add",
 *     "edit-form" = "/admin/accounting/workflow/{mc_workflow}/edit",
 *     "delete-form" = "/admin/accounting/workflow/{mc_workflow}/delete",
 *     "collection" = "/admin/accounting/workflow"
 *   },
 *   config_export = {
 *     "id",
 *     "label",
 *     "summary",
 *     "local",
 *     "creation",
 *     "direction",
 *     "states",
 *     "disabled"
 *   }
 * )
 * @todo make an edit form with a link to it and validation
 */
class Workflow extends ConfigEntityBase {

  const CREDIT = 'credit';
  const BILL = 'bill';
  const THIRDPARTY = '3rdparty';

  /**
   * Unique ID
   * @var string
   */
  public string $id;

  /**
   * Name to show in lists
   * @var string
   */
  public string $label;

  /**
   * Short description
   * @var string
   */
  public string $summary = '';

  /**
   * Whether this workflow originates locally
   * @var bool
   */
  public bool $local = TRUE;

  /**
   * Whether this is BILL or CREDIT or THIRDPARTY
   * @var string
   */
  public string $direction = '3rdparty';

  /**
   * Creation parameters.
   * @var stdClass with properties string state, and bool confirm
   */
  public $creation;

  /**
   * Structured data
   * @var array
   */
  public array $states = [];

  /**
   * TRUE if the workflow is disabled.
   * @var bool
   */
  public bool $disabled = FALSE;

  public static function loadAlphabetical($include_disabled = FALSE) {
    $all = static::loadMultiple();
    usort(
      $all,
      function ($a, $b){return $a->label <=> $b->label;}
    );
    if (!$include_disabled) {
      $all =  array_filter($all, function ($w) {return $w->disabled <> TRUE;});
    }
    return $all;
  }

}
