<?php

namespace Drupal\mcapi\Form;

use Drupal\mcapi\Entity\Transaction;
use Drupal\mcapi\Functions;
use Drupal\Core\Url;
use Drupal\Core\TempStore\PrivateTempStoreFactory;
use Drupal\Core\Entity\ContentEntityForm;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Form for adding transactions.
 */
class TransactionForm extends ContentEntityForm {

  /**
   * The tempStore service.
   *
   * @var Drupal\Core\TempStore\PrivateTempStore
   */
  protected $tempStore;

  /**
   * The request object.
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected $request;

  /**
   *
   * @param $entity_repository
   * @param $entity_type_bundle_info
   * @param $time
   * @param PrivateTempStoreFactory $tempstoreFactory
   * @param Request $current_request
   */
  public function __construct($entity_repository, $entity_type_bundle_info, $time, PrivateTempStoreFactory $tempstoreFactory, Request $current_request) {
    parent::__construct($entity_repository, $entity_type_bundle_info, $time);
    $this->tempStore = $tempstoreFactory->get('TransactionForm');
    $this->request = $current_request;
  }

  /**
   * {@inheritDoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity.repository'),
      $container->get('entity_type.bundle.info'),
      $container->get('datetime.time'),
      $container->get('tempstore.private'),
      $container->get('request_stack')->getCurrentRequest()
    );
  }

  /**
   * {@inheritDoc}
   *
   * @note we are overriding here because this form is neither for saving nor
   * deleting and because previewing is not optional.
   */
  protected function actions(array $form, FormStateInterface $form_state) {
    return [
      'submit' => [
        '#type' => 'submit',
        '#value' => $this->t('Preview'),
        // N.B. NOT ::save
        '#submit' => ['::submitForm'],
      ],
    ];
  }

  /**
   * {@inheritDoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->updateChangedTime($this->entity);
    // Retrieve the already built and validated transaction.
    $this->entity = Functions::getAssembledTransaction($this->entity);
    $this->entity->created->value = $this->time->getRequestTime();
    $creation_params = (object)$this->entity->workflow->entity->creation;
    if ($creation_params->confirm) {
      // Put the whole transaction in the userTempStore and go to the confirm page.
      $this->tempStore->set('mc_transaction', $this->entity);
      $redirect = Url::fromRoute(
        'entity.mc_transaction.transition',
        ['mc_transaction' => 0, 'dest_state' => $creation_params->state]
      );
    }
    else {
      $this->entity->save();
      \Drupal::messenger($this->t('The transaction was saved.'));
      $redirect = Url::fromRoute(
        'entity.mc_transaction.canonical',
        ['mc_transaction' => $this->entity->id()]
      );
    }
    $form_state->setRedirectUrl($redirect);
  }

  /**
   * {@inheritDoc}
   * @todo make this work for passing transaction properties.
   */
  public function getEntityFromRouteMatch(RouteMatchInterface $route_match, $entity_type_id) {
    $props['workflow'] = $this->request->attributes->get('mc_workflow');
    $props += $this->request->query->all();
    return Transaction::create($props);
  }

  /**
   * Provide an ID for the transaction form consistent with the keys of hook_transction_form_list().
   */
  function transactionFormId() {
    return 'basic_'.$this->entity->workflow->entity->id();
  }

}
