<?php

namespace Drupal\mcapi;

use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\mcapi\Entity\Workflow;

/**
 * Provides dynamic permissions for nodes of different types.
 */
class Permissions {
  use StringTranslationTrait;

  /**
   * Returns an array of transaction type permissions, one for each active workflow.
   *
   * @return array
   *   The node type permissions.
   *   @see \Drupal\user\PermissionHandlerInterface::getPermissions()
   */
  public function transactionTypePermissions() {
    $perms = [];
    foreach (Workflow::loadAlphabetical() as $wf) {
      $perms["create $wf->id transaction"] = [
        'title' => $this->t('Create %type transaction', ['%type' => $wf->label]),
        'description' => $wf->summary
      ];
    }
    return $perms;
  }

}
