<?php

namespace Drupal\mcapi\Plugin\Derivative;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Routing\RouteProviderInterface;
use Drupal\Core\Extension\ThemeHandlerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Component\Plugin\Derivative\DeriverBase;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Plugin\Discovery\ContainerDeriverInterface;
use Drupal\Core\Session\AccountInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a default implementation for menu link plugins.
 */
class ExtraLinks extends DeriverBase implements ContainerDeriverInterface {

  use StringTranslationTrait;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The route provider.
   *
   * @var \Drupal\Core\Routing\RouteProviderInterface
   */
  protected $routeProvider;

  /**
   * The theme handler.
   *
   * @var \Drupal\Core\Extension\ThemeHandlerInterface
   */
  protected $themeHandler;

  /**
   * The admin toolbar tools configuration.
   *
   * @var \Drupal\Core\Config\Config
   */
  protected $config;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * {@inheritdoc}
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, ModuleHandlerInterface $module_handler, RouteProviderInterface $route_provider, ThemeHandlerInterface $theme_handler, ConfigFactoryInterface $config_factory, AccountInterface $current_user) {
    $this->entityTypeManager = $entity_type_manager;
    $this->moduleHandler = $module_handler;
    $this->routeProvider = $route_provider;
    $this->themeHandler = $theme_handler;
    $this->config = $config_factory->get('admin_toolbar_tools.settings');
    $this->currentUser = $current_user;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, $base_plugin_id) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('module_handler'),
      $container->get('router.route_provider'),
      $container->get('theme_handler'),
      $container->get('config.factory'),
      $container->get('current_user')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getDerivativeDefinitions($base_plugin_definition) {
    $links = [];
    if (\Drupal::moduleHandler()->moduleExists('field_ui') && \Drupal::moduleHandler()->moduleExists('admin_toolbar_tools')) {
      // Adds common links to entities.
      foreach (['mc_transaction' => 'mcapi.admin.transaction.list', 'mc_wallet' => 'mcapi.admin_wallets'] as $entity_type_id => $parent_link_id) {

        $links["entity.$entity_type_id.field_ui_fields_"] = [
          'title' => $this->t('Manage fields'),
          'route_name' => "entity.$entity_type_id.field_ui_fields",
          'parent' => $parent_link_id,
          'weight' => 1,
        ] + $base_plugin_definition;
        $links["entity.entity_form_display.$entity_type_id.default_"] = [
          'title' => $this->t('Manage form display'),
          'route_name' => "entity.entity_form_display.$entity_type_id.default",
          'parent' => $parent_link_id,
          'weight' => 2,
        ] + $base_plugin_definition;
        $links["entity.entity_view_display.$entity_type_id.default_"] = [
          'title' => $this->t('Manage display'),
          'route_name' => "entity.entity_view_display.$entity_type_id.default",
          'parent' => $parent_link_id,
          'weight' => 3,
        ] + $base_plugin_definition;
      }
    }
    return $links;
  }

}
