<?php

namespace Drupal\mcapi\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\mcapi\Functions;

/**
 * Plugin implementation of the 'mc_entry_sentence' formatter.
 *
 * @FieldFormatter(
 *   id = "mc_entry_sentence",
 *   label = @Translation("Sentence"),
 *   description = @Translation("Display a transaction entry as a sentence."),
 *   field_types = {
 *     "entry"
 *   }
 * )
 */
class EntrySentenceFormatter extends FormatterBase {

  /**
   * {@inheritDoc}
   */
  public static function defaultSettings() {
    return [
      'template' => TRUE,
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritDoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $elements['template'] = [
      '#title' => $this->t('Template, using entry tokens'),
      '#description' => $this->t(
        'The following tokens are available: @tokens',
        ['@tokens' => '[payer], [payee], [description], [quant]']
      ),
      '#type' => 'textfield',
      '#default_value' => $this->getSetting('template')
    ];

    return $elements;
  }

  /**
   * {@inheritDoc}
   */
  public function settingsSummary() {
    return [t('Main entry is themed with mc_main_entry.<br />Additional entries shown as <br />@template', ['@template' => $this->getSetting('template')])];
  }

  /**
   * {@inheritDoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];
    // Only render the non-primary entries.
    foreach ($items as $entry) {
      $elements[] = [
        '#type' => 'inline_template',
        '#template' => $this->getSetting('template'),
        '#context' => [
          'payee' => $entry->payee->label(),
          'payer' => $entry->payer->label(),
          'quant' => Functions::formatCurrency($entry->quant),
          'description' => $entry->description
        ],
        '#mc_entry' => $entry,
      ];
    }
    return $elements;
  }

}
