<?php

namespace Drupal\mcapi\Plugin\Field\FieldType;

use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\Field\FieldItemBase;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\TypedData\DataReferenceTargetDefinition;

/**
 * Plugin implementation of the 'worth' field type.
 *
 * @FieldType(
 *   id = "mc_signature",
 *   label = @Translation("Signature"),
 *   description = @Translation("A signature needed and sign date"),
 *   category = @Translation("Community Accounting"),
 *   default_formatter = "mc_signatures",
 *   list_class = "\Drupal\mcapi\Plugin\Field\SignatureFieldItemList"
 * )
 */
class SignatureItem extends FieldItemBase {

  /**
   * {@inheritDoc}
   */
  public static function propertydefinitions(FieldStorageDefinitionInterface $field_definition) {
    $properties['uid'] = DataReferenceTargetDefinition::create('integer')
      ->setLabel(new TranslatableMarkup('User ID'))
      ->setDescription(new TranslatableMarkup('User who has to sign'))
      ->setRequired(TRUE);
    $properties['signed'] = DataReferenceTargetDefinition::create('integer')
      ->setLabel(new TranslatableMarkup('Signature date'))
      ->setDescription(new TranslatableMarkup('Unixtime of the signature, or 0 if not signed.'))
      ->setRequired(FALSE);
    // consider adding a computer field for the whole user.
    return $properties;
  }

  /**
   * {@inheritDoc}
   */
  public static function schema(FieldStorageDefinitionInterface $field_definition) {
    return [
      'columns' => [
        'uid' => [
          'description' => 'User ID',
          'type' => 'int',
          'size' => 'normal',
          'not null' => TRUE,
        ],
        'signed' => [
          'description' => 'wallet reference',
          'type' => 'int',
          'size' => 'normal',
          'not null' => TRUE,
        ]
      ]
    ];
  }

  /**
   * {@inheritDoc}
   * @note not used r/n because balance of trade wallet is 0
   */
  public function __isEmpty() {
    return empty($this->uid);
  }

  /**
   * {@inheritDoc}
   */
  public static function generateSampleValue(FieldDefinitionInterface $field_definition) {
    $wids = \Drupal::entityQuery('mc_wallet')->accessCheck(TRUE)->execute();
    shuffle($wids);
    return [
      'uid' => Wallet::load(array_pop($wids))->getOwner()->id(),
      'signed' => rand(0, 4) ? \Drupal::time()->getCurrentTime() : 0
    ];
  }

  /**
   * {@inheritDoc}
   */
  public static function mainPropertyName() {
    return 'signed';
  }

}
