<?php

namespace Drupal\mcapi\Plugin\Field\FieldWidget;

use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;

/**
 * Plugin implementation of the wallet selector widget.
 *
 * @FieldWidget(
 *   id = "mc_entry",
 *   label = @Translation("Entry"),
 *   description = @Translation("Enter the payer, payee, quant and description"),
 *   field_types = {
 *     "entry"
 *   },
 *   multiple_values = true
 * )
 */
class EntryWidget extends WidgetBase {

  /**
   * {@inheritDoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $elements = static::rawElements($items, $delta);
    $w = 0;
    foreach ($elements as &$e) {
      $e['#weight'] = $w++;
    }
    $element += $elements;
    return $element;
  }

  static function rawElements(FieldItemListInterface $items, $delta) {
    $direction = $items->getEntity()->workflow->entity->direction;
    return [
      'description' => [
        '#title' => t('Description'),
        '#type' => 'textfield',
        '#default_value' => $items[$delta]->value??'',
        '#weight' => 2
      ],
      'quant' => [
        '#title' => t('Amount'),
        '#type' => 'worth',
        '#default_value' => $items[$delta]->value??0,
        '#weight' => 4
      ],
      'payee_wid' => [
        '#title' => t('Payee'),
        '#type' => $direction == 'bill' ? 'my_wallet' : 'wallet_entity_auto',
        '#default_value' => $items[$delta]->target_id??'',
        '#placeholder' => t('Wallet name'),
        '#weight' => 6
      ],
      'payer_wid' => [
        '#title' => t('Payer'),
        '#type' => $direction == 'credit' ? 'my_wallet' : 'wallet_entity_auto',
        '#default_value' => $items[$delta]->target_id??'',
        '#placeholder' => t('Wallet name'),
        '#weight' => 8
      ]
    ];
  }

  /**
   * {@inheritDoc}
   */
  public function massageFormValues(array $values, array $form, FormStateInterface $form_state) {
    $values['primary'] = 1;
    if (!isset($values['metadata'])) {
      $values['metadata'] = [];
    }
    return $values;
  }

}
