<?php

namespace Drupal\mcapi\Plugin\Field\FieldWidget;

use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;

/**
 * Plugin implementation of the wallet selector widget.
 *
 * @FieldWidget(
 *   id = "mc_worth_nullable",
 *   label = @Translation("Worth"),
 *   description = @Translation("A quantity of community currency."),
 *   field_types = {"mc_worth"},
 *   multiple_values = false
 * )
 */
class Worth extends WidgetBase {

  /**
   * {@inheritDoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    return [
      'value' => $element + [
        '#type' => 'worth',
        '#default_value' => $items[$delta]->value,
        '#attached' => ['library' => ['mcapi_limits/minmax']],
        '#min' => 0,
      ],
      'reset' => [
        '#type' => 'button',
        '#name' => $items->getName(),
        '#value' => \Drupal\Core\Render\Markup::Create('<a href="#">'.t('Revert to default').'</a>'),
        '#ajax' => [
          'callback' => [$this, 'unsetPersonalLimits'],
          'wrapper' => 'edit-'.$items->getName().'-wrapper',
          'effect' => 'fade',
        ],
        // Make the button appear as a link.
        '#weight' => 2,
      ]
    ];
  }

  /**
   * @note This isn't really part of this form and the ajax that calls it isn't really form API.
   */
  function unsetPersonalLimits($form, FormStateInterface $form_state) {
    $field_name = $form_state->getTriggeringElement()['#name'];
    \Drupal::database()->delete('mc_wallet__'.$field_name)
      ->condition('entity_id', $form_state->getFormObject()->getEntity()->id())
      ->execute();
    unset($form[$field_name]['widget'][0]['reset']);
    $form[$field_name]['widget'][0]['value']['main']['#default_value'] = '';
    $form[$field_name]['widget'][0]['value']['main']['#value'] = '';
    return $form[$field_name];
  }


}
