<?php

namespace Drupal\mcapi\Plugin\Field;

use Drupal\Core\Field\FieldItemList;

/**
 * A signature field list
 */
class SignatureFieldItemList extends FieldItemList {

  /**
   * Sign for a given user.
   *
   * @param int $uid
   * @return void
   * @throws \Exception
   *
   * @note Does NOT save the transaction.
   */
  function sign(int $uid) : void {
    $values = $this->getValue();
    foreach ($values as &$val) {
      if ($val['uid'] == $uid) {
        $val['signed'] = \Drupal::time()->getRequestTime();
        $this->setValue($values);
        return;
      }
    }
    throw new \Exception("User $uid cannot sign transaction ".$this->getEntity()->id());
  }

  /**
   * Add all signatures.
   * @return void
   */
  function signAll() : void {
    $values = $this->getValue();
    $time = \Drupal::time()->getRequestTime();
    foreach ($values as &$val) {
      $val['signed'] = $time;
    }
    $this->setValue($values);
  }

  /**
   * Get the uids of the users who have yet to sign.
   * @return array
   */
  function getUnsigned() : array {
    $unsigned = [];
    foreach ($this->getValue() as $val) {
      if (empty($val['signed'])) {
        $unsigned[] = $val['uid'];
      }
    }
    return $unsigned;
  }

  function hasSigned(int $uid) : bool {
    foreach ($this->getValue() as $val) {
      if ($val['uid'] == $uid and !empty($val['signed'])) {
        return TRUE;
      }
    }
    return FALSE;
  }

  function removeSignature($uid) {
    foreach ($this->getValue() as $delta => $val) {
      if ($val['uid'] == $uid) {
        $this->removeItem($delta);
      }
    }
  }
  public function appendItem($value = NULL) {
    foreach ($this->list as $item) {
      if ($item->uid == $value['uid']) {
        return;
      }
    }
    return parent::appendItem($value);
  }
}
