<?php

namespace Drupal\mcapi\Plugin\migrate\destination;

use Drupal\mcapi\Entity\Transaction;
use Drupal\migrate\Row;
use Drupal\migrate\Plugin\migrate\destination\DestinationBase;
use Drupal\migrate\Plugin\MigrationInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Save the signatures into the transaction signatures field
 *
 * @MigrateDestination(
 *   id = "mc_signature"
 * )
 */
class Signature extends DestinationBase implements ContainerFactoryPluginInterface {

  protected $supportsRollback = TRUE;

  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition, MigrationInterface $migration = NULL): static {
    return new static($configuration, $plugin_id, $plugin_definition, $migration);
  }

  /**
   * {@inheritDoc}
   */
  public function import(Row $row, array $old_destination_id_values = array()) {
    $dest = $row->getDestination();
    $transaction = Transaction::load($dest['xid']);
    $transaction->signatures->appendItem($dest);
    $transaction->save();
    return [$row->getDestinationProperty('xid'), $row->getDestinationProperty('uid')];
  }

  /**
   * {@inheritDoc}
   */
  public function fields(MigrationInterface $migration = NULL) {
    return ['xid', 'uid', 'signed'];
  }

  /**
   * {@inheritDoc}
   */
  public function getIds() {
    return [
      'xid' => ['type' => 'string'],
      'uid' => ['type' => 'integer']
    ];
  }

  /**
   * {@inheritDoc}
   */
  public function rollback(array $destination_identifier) {
    $this->database->delete('mc_transaction__signatures')
      ->condition('signatures_uid', $destination_identifier['uid'])
      ->condition('signatures_signed', $destination_identifier['xid'])
      ->execute();

    $this->database->delete('mc_transaction_revision__signatures 	')
      ->condition('signatures_uid', $destination_identifier['uid'])
      ->condition('signatures_signed', $destination_identifier['xid'])
      ->execute();
  }


}
