<?php
namespace Drupal\mcapi\Plugin\migrate\process;

use Drupal\migrate\MigrateExecutableInterface;
use Drupal\migrate\ProcessPluginBase;
use Drupal\migrate\Row;

/**
 * Change the meaning of this field from TRUE = pending to int = date signed,
 * using the transaction changed date as default.
 *
 * @MigrateProcessPlugin(
 *   id = "mc_signature"
 * )
 */
class Signature extends ProcessPluginBase {

  /**
   * {@inheritDoc}
   * $value 1 or 0 meaning that the transaction is pending or not.
   */
  public function transform($pending, MigrateExecutableInterface $migrate_executable, Row $row, $destination_property) {
    $value =  [
      'uid' => $row->getSourceProperty('uid'),
      'xid' => $row->getSourceProperty('serial')
    ];
    if ($pending) {
      $value['signed'] = 0;
    }
    else {
      // If the pending field was 0, then we replace it with the created date
      $value['signed'] = $row->getSourceProperty('created');
      // Pretend that all transactions were signed one day after they were created
      if ($row->getSourceProperty('uid') != $row->getSourceProperty('creator')) {
        $value['signed'] += 86400;
      }
    }
    return $value;
  }

}
